unit hiPIPEServer;

interface

uses Kol,Share,Windows,Debug;

const
  IN_BUF_SIZE = MAXWORD;
  OUT_BUF_SIZE = MAXWORD;
  TIME_OUT = 0;
  MAX_READ = MAXWORD;
  MAX_PIPE_INSTANCES = 100;
  NAME_SIZE = 25;
  LINE_LEN = 80;

type
  THIPIPEServer = class(TDebug)
   private
    BufSize:cardinal;
    fBufferSize:real;
    hPipe:cardinal;
    lol:string;
    th:PThread;
    str:string;
    Strm: PStream;
    Connect:boolean;

    function _OnExec(Sender:PThread):Integer;
    procedure Stop;
    procedure SyncExec;
   public
    _prop_Name:string;
    _data_Name:THI_Event;

    _event_onStatus:THI_Event;
    _event_onRead:THI_Event;

    property _prop_BufferSize:real write fBufferSize;
    destructor Destroy; override;
    procedure _work_doCreate(var _Data:TData; Index:word);
    procedure _work_doDestroy(var _Data:TData; Index:word);
    procedure _work_doBufferSize(var _Data:TData; Index:word);
  end;

implementation

destructor THIPIPEServer.Destroy;
begin
   Stop;
   inherited;
end;

procedure THIPIPEServer._work_doCreate;
begin
   if not Assigned(th) then begin
      if fBufferSize < 0.25 then BufSize := 256 else BufSize := Round(fBufferSize*1024);  
      lol:= ReadString(_Data,_data_Name,_prop_Name);
      hPipe := CreateNamedPipe(PChar('\\.\PIPE\'+lol), //  
               PIPE_ACCESS_INBOUND or     //   -
//               PIPE_ACCESS_DUPLEX or      //  
               FILE_FLAG_OVERLAPPED,      //  -
               PIPE_WAIT or               //  
               PIPE_READMODE_BYTE or      //      
               PIPE_TYPE_BYTE,
               PIPE_UNLIMITED_INSTANCES,        //    .
//               OUT_BUF_SIZE,              //   /.
//               IN_BUF_SIZE,
               0,
               BufSize,
               TIME_OUT,                  // -.
               nil);                      //  .

      if hPipe = INVALID_HANDLE_VALUE then
         _hi_OnEvent(_event_onStatus,0)
      else begin
         Connect := false; 
         _hi_OnEvent(_event_onStatus,1);
         {$ifdef F_P}
         th := NewThreadforFPC;
         {$else}
         th := NewThread;
         {$endif}
         th.OnExecute := _OnExec;
         th.Resume;
      end;
   end;
end;

function THIPIPEServer._OnExec;
var   nBytesRead: Cardinal;
      OverLapRd: OVERLAPPED;
      hEventRd: THANDLE;
      rc: Boolean;
      LastErr: DWORD;
begin
   while not Sender.Terminated do begin
      Connect := ConnectNamedPipe(hPipe, nil);
      SetLength(str,BufSize);
      hEventRd := CreateEvent (nil, true, false, nil);
      FillChar(OverLapRd, sizeof(OVERLAPPED), 0);
      OverLapRd.hEvent := hEventRd;
      if Assigned(Strm) then free_and_nil(Strm);
      Strm := NewMemoryStream;
      repeat
         rc := ReadFile(hPipe, str[1], BufSize, nBytesRead, @OverLapRd);
         if not rc then begin
            lastErr := GetLastError;
            case lastErr of
               ERROR_IO_PENDING: //   
                  WaitForSingleObject (hEventRd, INFINITE);
               else nBytesRead := 0;             
            end;
         end else if nBytesRead <> 0 then Strm.Write(str[1],nBytesRead);   
      until nBytesRead = 0;
      if Strm.Size <> 0 then begin
         Strm.Position := 0; 
         Sender.Synchronize(SyncExec);
      end;
      SetLength(str,0);
      DisconnectNamedPipe(hPipe);
      Connect := false;
   end;
   Result := 0;
end;

procedure THIPIPEServer.SyncExec;
begin
   _hi_OnEvent(_event_onRead,strm);
   free_and_nil(Strm);
end;

procedure THIPIPEServer.Stop;
begin
   if Assigned(th) then begin
      SetLength(str,0);
      If Connect then DisconnectNamedPipe(hPipe);
      CloseHandle(hPipe);
      free_and_nil(th);
      free_and_nil(Strm);
   end;   
end;

procedure THIPIPEServer._work_doDestroy;
begin
   Stop;
   inherited;
end;

procedure THIPIPEServer._work_doBufferSize;
begin
   fBufferSize := ToReal(_Data);
end;

end.